@@ -21,15 +21,15 @@ from dask.utils import raises, ignoring, tmpfile, tmpdir
 from dask.utils_test import inc
 
 from dask.array import chunk
-from dask.array.core import (getem, getarray, top, dotmany, concatenate3,
-                             broadcast_dimensions, Array, stack, concatenate,
-                             from_array, take, elemwise, isnull, notnull,
-                             broadcast_shapes, partial_by_order, exp,
+from dask.array.core import (getem, getarray, getarray_nofancy, top, dotmany,
+                             concatenate3, broadcast_dimensions, Array, stack,
+                             concatenate, from_array, take, elemwise, isnull,
+                             notnull, broadcast_shapes, partial_by_order, exp,
                              tensordot, choose, where, coarsen, insert,
                              broadcast_to, reshape, fromfunction,
                              blockdims_from_blockshape, store, optimize,
                              from_func, normalize_chunks, broadcast_chunks,
-                             atop, from_delayed)
+                             atop, from_delayed, concatenate_axes)
 from dask.array.utils import assert_eq
 
 # temporary until numpy functions migrated
@@ -51,11 +51,12 @@ def same_keys(a, b):
 
 
 def test_getem():
-    assert getem('X', (2, 3), shape=(4, 6)) == \
-    {('X', 0, 0): (getarray, 'X', (slice(0, 2), slice(0, 3))),
-     ('X', 1, 0): (getarray, 'X', (slice(2, 4), slice(0, 3))),
-     ('X', 1, 1): (getarray, 'X', (slice(2, 4), slice(3, 6))),
-     ('X', 0, 1): (getarray, 'X', (slice(0, 2), slice(3, 6)))}
+    for fancy, getter in [(True, getarray), (False, getarray_nofancy)]:
+        sol = {('X', 0, 0): (getter, 'X', (slice(0, 2), slice(0, 3))),
+               ('X', 1, 0): (getter, 'X', (slice(2, 4), slice(0, 3))),
+               ('X', 1, 1): (getter, 'X', (slice(2, 4), slice(3, 6))),
+               ('X', 0, 1): (getter, 'X', (slice(0, 2), slice(3, 6)))}
+        assert getem('X', (2, 3), shape=(4, 6), fancy=fancy) == sol
 
 
 def test_top():
@@ -66,14 +67,14 @@ def test_top():
          ('z', 1, 1): (inc, ('x', 1, 1))}
 
     assert top(add, 'z', 'ij', 'x', 'ij', 'y', 'ij',
-                numblocks={'x': (2, 2), 'y': (2, 2)}) == \
+               numblocks={'x': (2, 2), 'y': (2, 2)}) == \
         {('z', 0, 0): (add, ('x', 0, 0), ('y', 0, 0)),
          ('z', 0, 1): (add, ('x', 0, 1), ('y', 0, 1)),
          ('z', 1, 0): (add, ('x', 1, 0), ('y', 1, 0)),
          ('z', 1, 1): (add, ('x', 1, 1), ('y', 1, 1))}
 
     assert top(dotmany, 'z', 'ik', 'x', 'ij', 'y', 'jk',
-                    numblocks={'x': (2, 2), 'y': (2, 2)}) == \
+               numblocks={'x': (2, 2), 'y': (2, 2)}) == \
         {('z', 0, 0): (dotmany, [('x', 0, 0), ('x', 0, 1)],
                                 [('y', 0, 0), ('y', 1, 0)]),
          ('z', 0, 1): (dotmany, [('x', 0, 0), ('x', 0, 1)],
@@ -90,7 +91,7 @@ def test_top():
 
 def test_top_supports_broadcasting_rules():
     assert top(add, 'z', 'ij', 'x', 'ij', 'y', 'ij',
-                numblocks={'x': (1, 2), 'y': (2, 1)}) == \
+               numblocks={'x': (1, 2), 'y': (2, 1)}) == \
         {('z', 0, 0): (add, ('x', 0, 0), ('y', 0, 0)),
          ('z', 0, 1): (add, ('x', 0, 1), ('y', 0, 0)),
          ('z', 1, 0): (add, ('x', 0, 0), ('y', 1, 0)),
@@ -187,7 +188,7 @@ def test_numblocks_suppoorts_singleton_block_dims():
     dsk = merge({name: 'some-array'}, getem(name, shape=shape, chunks=chunks))
     a = Array(dsk, name, chunks, shape=shape)
 
-    assert set(concat(a._keys())) == set([('x', i, 0) for i in range(100//10)])
+    assert set(concat(a._keys())) == set([('x', i, 0) for i in range(100 // 10)])
 
 
 def test_keys():
@@ -493,6 +494,16 @@ def test_field_access():
     assert same_keys(y[['b', 'a']], y[['b', 'a']])
 
 
+def test_field_access_with_shape():
+    dtype = [('col1', ('f4', (3, 2))), ('col2', ('f4', 3))]
+    data = np.ones((100, 50), dtype=dtype)
+    x = da.from_array(data, 10)
+    assert_eq(x['col1'], data['col1'])
+    assert_eq(x[['col1']], data[['col1']])
+    assert_eq(x['col2'], data['col2'])
+    assert_eq(x[['col1', 'col2']], data[['col1', 'col2']])
+
+
 def test_tensordot():
     x = np.arange(400).reshape((20, 20))
     a = from_array(x, chunks=(5, 4))
@@ -587,7 +598,7 @@ def test_coarsen_with_excess():
     assert_eq(coarsen(np.min, x, {0: 3}, trim_excess=True),
               np.array([0, 5]))
     assert_eq(coarsen(np.sum, x, {0: 3}, trim_excess=True),
-              np.array([0+1+2, 5+6+7]))
+              np.array([0 + 1 + 2, 5 + 6 + 7]))
 
 
 def test_insert():
@@ -816,6 +827,7 @@ def test_repr():
     assert str(d.shape) in repr(d)
     assert str(d._dtype) in repr(d)
 
+
 def test_slicing_with_ellipsis():
     x = np.arange(256).reshape((4, 4, 4, 4))
     d = da.from_array(x, chunks=((2, 2, 2, 2)))
@@ -843,8 +855,8 @@ def test_dtype():
 def test_blockdims_from_blockshape():
     assert blockdims_from_blockshape((10, 10), (4, 3)) == ((4, 4, 2), (3, 3, 3, 1))
     assert raises(TypeError, lambda: blockdims_from_blockshape((10,), None))
-    assert blockdims_from_blockshape((1e2, 3), [1e1, 3]) == ((10,)*10, (3,))
-    assert blockdims_from_blockshape((np.int8(10),), (5,)) == ((5, 5),)
+    assert blockdims_from_blockshape((1e2, 3), [1e1, 3]) == ((10, ) * 10, (3, ))
+    assert blockdims_from_blockshape((np.int8(10), ), (5, )) == ((5, 5), )
 
 
 def test_coerce():
@@ -1151,17 +1163,17 @@ def test_arithmetic():
     assert_eq(da.sin(a), np.sin(x))
     assert_eq(da.cos(b), np.cos(y))
     assert_eq(da.tan(a), np.tan(x))
-    assert_eq(da.arcsin(b/10), np.arcsin(y/10))
-    assert_eq(da.arccos(b/10), np.arccos(y/10))
-    assert_eq(da.arctan(b/10), np.arctan(y/10))
-    assert_eq(da.arctan2(b*10, a), np.arctan2(y*10, x))
+    assert_eq(da.arcsin(b / 10), np.arcsin(y / 10))
+    assert_eq(da.arccos(b / 10), np.arccos(y / 10))
+    assert_eq(da.arctan(b / 10), np.arctan(y / 10))
+    assert_eq(da.arctan2(b * 10, a), np.arctan2(y * 10, x))
     assert_eq(da.hypot(b, a), np.hypot(y, x))
     assert_eq(da.sinh(a), np.sinh(x))
     assert_eq(da.cosh(b), np.cosh(y))
     assert_eq(da.tanh(a), np.tanh(x))
-    assert_eq(da.arcsinh(b*10), np.arcsinh(y*10))
-    assert_eq(da.arccosh(b*10), np.arccosh(y*10))
-    assert_eq(da.arctanh(b/10), np.arctanh(y/10))
+    assert_eq(da.arcsinh(b * 10), np.arcsinh(y * 10))
+    assert_eq(da.arccosh(b * 10), np.arccosh(y * 10))
+    assert_eq(da.arctanh(b / 10), np.arctanh(y / 10))
     assert_eq(da.deg2rad(a), np.deg2rad(x))
     assert_eq(da.rad2deg(a), np.rad2deg(x))
 
@@ -1203,6 +1215,7 @@ def test_arithmetic():
     assert_eq((a + 1j * b).conj(), (x + 1j * y).conj())
 
     assert_eq(da.clip(b, 1, 4), np.clip(y, 1, 4))
+    assert_eq(b.clip(1, 4), y.clip(1, 4))
     assert_eq(da.fabs(b), np.fabs(y))
     assert_eq(da.sign(b - 2), np.sign(y - 2))
     assert_eq(da.absolute(b - 2), np.absolute(y - 2))
@@ -1221,6 +1234,18 @@ def test_arithmetic():
     assert_eq(da.around(a, -1), np.around(x, -1))
 
 
+def test_clip():
+    x = np.random.normal(0, 10, size=(10, 10))
+    d = da.from_array(x, chunks=(3, 4))
+
+    assert_eq(x.clip(5), d.clip(5))
+    assert_eq(x.clip(1, 5), d.clip(1, 5))
+    assert_eq(x.clip(min=5), d.clip(min=5))
+    assert_eq(x.clip(max=5), d.clip(max=5))
+    assert_eq(x.clip(max=1, min=5), d.clip(max=1, min=5))
+    assert_eq(x.clip(min=1, max=5), d.clip(min=1, max=5))
+
+
 def test_elemwise_consistent_names():
     a = da.from_array(np.arange(5, dtype='f4'), chunks=(2,))
     b = da.from_array(np.arange(5, dtype='f4'), chunks=(2,))
@@ -1396,6 +1421,7 @@ def test_bincount_raises_informative_error_on_missing_minlength_kwarg():
     else:
         assert False
 
+
 @pytest.mark.skipif(LooseVersion(np.__version__) < '1.10.0',
                     reason="NumPy doesn't yet support nd digitize")
 def test_digitize():
@@ -1521,19 +1547,18 @@ def test_map_blocks3():
     d = da.from_array(x, chunks=5)
     e = da.from_array(y, chunks=5)
 
-    assert_eq(da.core.map_blocks(lambda a, b: a+2*b, d, e, dtype=d.dtype),
-              x + 2*y)
+    assert_eq(da.core.map_blocks(lambda a, b: a + 2 * b, d, e, dtype=d.dtype),
+              x + 2 * y)
 
     z = np.arange(100).reshape((10, 10))
     f = da.from_array(z, chunks=5)
 
-    func = lambda a, b: a + 2*b
+    func = lambda a, b: a + 2 * b
     res = da.core.map_blocks(func, d, f, dtype=d.dtype)
-    assert_eq(res, x + 2*z)
+    assert_eq(res, x + 2 * z)
     assert same_keys(da.core.map_blocks(func, d, f, dtype=d.dtype), res)
 
-    assert_eq(da.map_blocks(func, f, d, dtype=d.dtype),
-              z + 2*x)
+    assert_eq(da.map_blocks(func, f, d, dtype=d.dtype), z + 2 * x)
 
 
 def test_from_array_with_missing_chunks():
@@ -1637,7 +1662,7 @@ def test_point_slicing():
 
 def test_point_slicing_with_full_slice():
     from dask.array.core import _vindex_transpose, _get_axis
-    x = np.arange(4*5*6*7).reshape((4, 5, 6, 7))
+    x = np.arange(4 * 5 * 6 * 7).reshape((4, 5, 6, 7))
     d = da.from_array(x, chunks=(2, 3, 3, 4))
 
     inds = [
@@ -1761,7 +1786,7 @@ def test_memmap():
 
 
 def test_to_npy_stack():
-    x = np.arange(5*10*10).reshape((5, 10, 10))
+    x = np.arange(5 * 10 * 10).reshape((5, 10, 10))
     d = da.from_array(x, chunks=(2, 4, 4))
 
     with tmpdir() as dirname:
@@ -2004,8 +2029,8 @@ def test_diag():
 
 def test_tril_triu():
     A = np.random.randn(20, 20)
-    for chunk in [5, 4]:
-        dA = da.from_array(A, (chunk, chunk))
+    for chk in [5, 4]:
+        dA = da.from_array(A, (chk, chk))
 
         assert np.allclose(da.triu(dA).compute(), np.triu(A))
         assert np.allclose(da.tril(dA).compute(), np.tril(A))
@@ -2015,6 +2040,7 @@ def test_tril_triu():
             assert np.allclose(da.triu(dA, k).compute(), np.triu(A, k))
             assert np.allclose(da.tril(dA, k).compute(), np.tril(A, k))
 
+
 def test_tril_triu_errors():
     A = np.random.random_integers(0, 10, (10, 10, 10))
     dA = da.from_array(A, chunks=(5, 5, 5))
@@ -2031,6 +2057,34 @@ def test_atop_names():
     assert y.name.startswith('add')
 
 
+def test_atop_new_axes():
+    def f(x):
+        return x[:, None] * np.ones((1, 7))
+    x = da.ones(5, chunks=2)
+    y = atop(f, 'aq', x, 'a', new_axes={'q': 7}, concatenate=True,
+             dtype=x._dtype)
+    assert y.chunks == ((2, 2, 1), (7,))
+    assert_eq(y, np.ones((5, 7)))
+
+    def f(x):
+        return x[None, :] * np.ones((7, 1))
+    x = da.ones(5, chunks=2)
+    y = atop(f, 'qa', x, 'a', new_axes={'q': 7}, concatenate=True,
+             dtype=x._dtype)
+    assert y.chunks == ((7,), (2, 2, 1))
+    assert_eq(y, np.ones((7, 5)))
+
+    def f(x):
+        y = x.sum(axis=1)
+        return y[:, None] * np.ones((1, 5))
+
+    x = da.ones((4, 6), chunks=(2, 2))
+    y = atop(f, 'aq', x, 'ab', new_axes={'q': 5}, concatenate=True,
+             dtype=x._dtype)
+    assert y.chunks == ((2, 2), (5,))
+    assert_eq(y, np.ones((4, 5)) * 6)
+
+
 def test_atop_kwargs():
     def f(a, b=0):
         return a + b
@@ -2093,3 +2147,88 @@ def test_array_picklable():
     a = da.arange(100, chunks=25)
     a2 = loads(dumps(a))
     assert_eq(a, a2)
+
+
+def test_swapaxes():
+    x = np.random.normal(0, 10, size=(10, 12, 7))
+    d = da.from_array(x, chunks=(4, 5, 2))
+
+    assert_eq(np.swapaxes(x, 0, 1), da.swapaxes(d, 0, 1))
+    assert_eq(np.swapaxes(x, 2, 1), da.swapaxes(d, 2, 1))
+    assert_eq(x.swapaxes(2, 1), d.swapaxes(2, 1))
+    assert_eq(x.swapaxes(0, 0), d.swapaxes(0, 0))
+    assert_eq(x.swapaxes(1, 2), d.swapaxes(1, 2))
+
+    assert d.swapaxes(0, 1).name == d.swapaxes(0, 1).name
+    assert d.swapaxes(0, 1).name != d.swapaxes(1, 0).name
+
+
+def test_from_array_raises_on_bad_chunks():
+    x = np.ones(10)
+
+    with pytest.raises(ValueError):
+        da.from_array(x, chunks=(5, 5, 5))
+
+    # with pytest.raises(ValueError):
+    #      da.from_array(x, chunks=100)
+
+    with pytest.raises(ValueError):
+        da.from_array(x, chunks=((5, 5, 5),))
+
+
+def test_concatenate_axes():
+    x = np.ones((2, 2, 2))
+
+    assert_eq(concatenate_axes([x, x], axes=[0]),
+              np.ones((4, 2, 2)))
+    assert_eq(concatenate_axes([x, x, x], axes=[0]),
+              np.ones((6, 2, 2)))
+    assert_eq(concatenate_axes([x, x], axes=[1]),
+              np.ones((2, 4, 2)))
+    assert_eq(concatenate_axes([[x, x], [x, x]], axes=[0, 1]),
+              np.ones((4, 4, 2)))
+    assert_eq(concatenate_axes([[x, x], [x, x]], axes=[0, 2]),
+              np.ones((4, 2, 4)))
+    assert_eq(concatenate_axes([[x, x, x], [x, x, x]], axes=[1, 2]),
+              np.ones((2, 4, 6)))
+
+    with pytest.raises(ValueError):
+        concatenate_axes([[x, x], [x, x]], axes=[0])  # not all nested lists accounted for
+    with pytest.raises(ValueError):
+        concatenate_axes([x, x], axes=[0, 1, 2, 3])  # too many axes
+
+
+def test_atop_concatenate():
+    x = da.ones((4, 4, 4), chunks=(2, 2, 2))
+    y = da.ones((4, 4), chunks=(2, 2))
+
+    def f(a, b):
+        assert isinstance(a, np.ndarray)
+        assert isinstance(b, np.ndarray)
+
+        assert a.shape == (2, 4, 4)
+        assert b.shape == (4, 4)
+
+        return (a + b).sum(axis=(1, 2))
+
+    z = atop(f, 'i', x, 'ijk', y, 'jk', concatenate=True, dtype=x._dtype)
+    assert_eq(z, np.ones(4) * 32)
+
+    z = atop(add, 'ij', y, 'ij', y, 'ij', concatenate=True, dtype=x._dtype)
+    assert_eq(z, np.ones((4, 4)) * 2)
+
+
+    def f(a, b, c):
+        assert isinstance(a, np.ndarray)
+        assert isinstance(b, np.ndarray)
+        assert isinstance(c, np.ndarray)
+
+        assert a.shape == (4, 2, 4)
+        assert b.shape == (4, 4)
+        assert c.shape == (4, 2)
+
+        return np.ones(5)
+
+    z = atop(f, 'j', x, 'ijk', y, 'ki', y, 'ij', concatenate=True,
+             dtype=x._dtype)
+    assert_eq(z, np.ones(10))
