@@ -1,7 +1,7 @@
 from __future__ import print_function, division, absolute_import
 
 from io import BytesIO
-from warnings import warn
+from warnings import warn, catch_warnings, simplefilter
 
 try:
     import psutil
@@ -68,8 +68,9 @@ def coerce_dtypes(df, dtypes):
             if (np.issubdtype(df.dtypes[c], np.floating) and
                     np.issubdtype(dtypes[c], np.integer)):
                 if (df[c] % 1).any():
-                    raise TypeError("Runtime type mismatch. "
-                    "Add {'%s': float} to dtype= keyword in read_csv" % c)
+                    msg = ("Runtime type mismatch. "
+                           "Add {'%s': float} to dtype= keyword in read_csv")
+                    raise TypeError(msg % c)
             df[c] = df[c].astype(dtypes[c])
 
 
@@ -128,16 +129,18 @@ def auto_blocksize(total_memory, cpu_count):
 
 # guess blocksize if psutil is installed or use acceptable default one if not
 if psutil is not None:
-    TOTAL_MEM = psutil.virtual_memory().total
-    CPU_COUNT = psutil.cpu_count()
-    AUTO_BLOCKSIZE = auto_blocksize(TOTAL_MEM, CPU_COUNT)
+    with catch_warnings():
+        simplefilter("ignore", RuntimeWarning)
+        TOTAL_MEM = psutil.virtual_memory().total
+        CPU_COUNT = psutil.cpu_count()
+        AUTO_BLOCKSIZE = auto_blocksize(TOTAL_MEM, CPU_COUNT)
 else:
     AUTO_BLOCKSIZE = 2**25
 
 
-def read_csv(urlpath, blocksize=AUTO_BLOCKSIZE, chunkbytes=None,
-        collection=True, lineterminator=None, compression=None,
-        sample=256000, enforce=False, storage_options=None, **kwargs):
+def read_csv(urlpath, blocksize=AUTO_BLOCKSIZE, collection=True,
+             lineterminator=None, compression=None, sample=256000,
+             enforce=False, storage_options=None, **kwargs):
     """ Read CSV files into a Dask.DataFrame
 
     This parallelizes the ``pandas.read_csv`` file in the following ways:
@@ -190,15 +193,16 @@ def read_csv(urlpath, blocksize=AUTO_BLOCKSIZE, chunkbytes=None,
         kwargs['lineterminator'] = lineterminator
     else:
         lineterminator = '\n'
-    if chunkbytes is not None:
-        warn("Deprecation warning: chunksize csv keyword renamed to blocksize")
-        blocksize = chunkbytes
     if 'index' in kwargs or 'index_col' in kwargs:
         raise ValueError("Keyword 'index' not supported "
                          "dd.read_csv(...).set_index('my-index') instead")
     for kw in ['iterator', 'chunksize']:
         if kw in kwargs:
             raise ValueError("%s not supported for dd.read_csv" % kw)
+    if kwargs.get('nrows', None):
+        raise ValueError("The 'nrows' keyword is not supported by "
+                         "`dd.read_csv`. To achieve the same behavior, it's "
+                         "recommended to use `dd.read_csv(...).head(n=nrows)`")
     if isinstance(kwargs.get('skiprows'), list):
         raise TypeError("List of skiprows not supported for dd.read_csv")
     if isinstance(kwargs.get('header'), list):
@@ -206,7 +210,7 @@ def read_csv(urlpath, blocksize=AUTO_BLOCKSIZE, chunkbytes=None,
 
     if blocksize and compression not in seekable_files:
         warn("Warning %s compression does not support breaking apart files\n"
-             "Please ensure that each individiaul file can fit in memory and\n"
+             "Please ensure that each individual file can fit in memory and\n"
              "use the keyword ``blocksize=None to remove this message``\n"
              "Setting ``blocksize=None``" % compression)
         blocksize = None
@@ -220,16 +224,15 @@ def read_csv(urlpath, blocksize=AUTO_BLOCKSIZE, chunkbytes=None,
                                          sample=sample,
                                          compression=compression,
                                          **(storage_options or {}))
+
     if not isinstance(values[0], (tuple, list)):
         values = [values]
 
-    if 'nrows' in kwargs:
-        values = [[values[0][0]]]
-
     if kwargs.get('header', 'infer') is None:
         header = b''
     else:
         header = sample.split(b_lineterminator)[0] + b_lineterminator
+
     head = pd.read_csv(BytesIO(sample), **kwargs)
 
     df = read_csv_from_bytes(values, header, head, kwargs,
