@@ -8,19 +8,19 @@ import dask
 import pytest
 from distutils.version import LooseVersion
 from threading import Lock
-import shutil
 from time import sleep
 import threading
 
 import dask.array as da
 import dask.dataframe as dd
-from dask.dataframe.io import (from_array, from_bcolz, from_dask_array)
+from dask.dataframe.io import from_array, from_dask_array
 from dask.delayed import Delayed
 
 from dask.utils import filetext, filetexts, tmpfile, tmpdir, dependency_depth
 from dask.async import get_sync
 
 from dask.dataframe.utils import eq
+from dask import compute
 
 ########
 # CSVS #
@@ -47,9 +47,8 @@ Edith,600
 
 def test_read_csv():
     with filetext(text) as fn:
-        f = dd.read_csv(fn, chunkbytes=30, lineterminator=os.linesep)
+        f = dd.read_csv(fn, blocksize=30, lineterminator=os.linesep)
         assert list(f.columns) == ['name', 'amount']
-        assert f._known_dtype
         result = f.compute(get=dask.get)
         # index may be different
         assert eq(result.reset_index(drop=True),
@@ -62,8 +61,7 @@ def test_read_multiple_csv():
             f.write(text)
         with open('_foo.2.csv', 'w') as f:
             f.write(text)
-        df = dd.read_csv('_foo.*.csv', chunkbytes=30)
-        assert df._known_dtype
+        df = dd.read_csv('_foo.*.csv', blocksize=30)
         assert df.npartitions > 2
 
         assert (len(dd.read_csv('_foo.*.csv').compute()) ==
@@ -90,9 +88,8 @@ def test_consistent_dtypes():
     """)
 
     with filetext(text) as fn:
-        df = dd.read_csv(fn, chunkbytes=30)
+        df = dd.read_csv(fn, blocksize=30)
         assert isinstance(df.amount.sum().compute(), float)
-        assert df._known_dtype
 
 datetime_csv_file = """
 name,amount,when
@@ -105,8 +102,7 @@ Dan,400,2014-01-01
 
 def test_read_csv_index():
     with filetext(text) as fn:
-        f = dd.read_csv(fn, chunkbytes=20).set_index('amount')
-        assert f._known_dtype
+        f = dd.read_csv(fn, blocksize=20).set_index('amount')
         result = f.compute(get=get_sync)
         assert result.index.name == 'amount'
 
@@ -123,7 +119,7 @@ def test_read_csv_index():
 
 def test_usecols():
     with filetext(datetime_csv_file) as fn:
-        df = dd.read_csv(fn, chunkbytes=30, usecols=['when', 'amount'])
+        df = dd.read_csv(fn, blocksize=30, usecols=['when', 'amount'])
         expected = pd.read_csv(fn, usecols=['when', 'amount'])
         assert (df.compute().values == expected.values).all()
 
@@ -133,86 +129,81 @@ def test_usecols():
 ####################
 
 
-def test_dummy_from_array():
+def test_meta_from_array():
     x = np.array([[1, 2], [3, 4]], dtype=np.int64)
-    res = dd.io._dummy_from_array(x)
+    res = dd.io._meta_from_array(x)
     assert isinstance(res, pd.DataFrame)
     assert res[0].dtype == np.int64
     assert res[1].dtype == np.int64
     tm.assert_index_equal(res.columns, pd.Index([0, 1]))
 
     x = np.array([[1., 2.], [3., 4.]], dtype=np.float64)
-    res = dd.io._dummy_from_array(x, columns=['a', 'b'])
+    res = dd.io._meta_from_array(x, columns=['a', 'b'])
     assert isinstance(res, pd.DataFrame)
     assert res['a'].dtype == np.float64
     assert res['b'].dtype == np.float64
     tm.assert_index_equal(res.columns, pd.Index(['a', 'b']))
 
-    msg = r"""Length mismatch: Expected axis has 2 elements, new values have 3 elements"""
-    with tm.assertRaisesRegexp(ValueError, msg):
-        dd.io._dummy_from_array(x, columns=['a', 'b', 'c'])
+    with pytest.raises(ValueError):
+        dd.io._meta_from_array(x, columns=['a', 'b', 'c'])
 
     np.random.seed(42)
     x = np.random.rand(201, 2)
     x = from_array(x, chunksize=50, columns=['a', 'b'])
-    assert len(x.divisions) == 6 # Should be 5 partitions and the end
+    assert len(x.divisions) == 6   # Should be 5 partitions and the end
 
 
-def test_dummy_from_1darray():
+def test_meta_from_1darray():
     x = np.array([1., 2., 3.], dtype=np.float64)
-    res = dd.io._dummy_from_array(x)
+    res = dd.io._meta_from_array(x)
     assert isinstance(res, pd.Series)
     assert res.dtype == np.float64
 
     x = np.array([1, 2, 3], dtype=np.object_)
-    res = dd.io._dummy_from_array(x, columns='x')
+    res = dd.io._meta_from_array(x, columns='x')
     assert isinstance(res, pd.Series)
     assert res.name == 'x'
     assert res.dtype == np.object_
 
     x = np.array([1, 2, 3], dtype=np.object_)
-    res = dd.io._dummy_from_array(x, columns=['x'])
+    res = dd.io._meta_from_array(x, columns=['x'])
     assert isinstance(res, pd.DataFrame)
     assert res['x'].dtype == np.object_
     tm.assert_index_equal(res.columns, pd.Index(['x']))
 
-    msg = r"""Length mismatch: Expected axis has 1 elements, new values have 2 elements"""
-    with tm.assertRaisesRegexp(ValueError, msg):
-        dd.io._dummy_from_array(x, columns=['a', 'b'])
+    with pytest.raises(ValueError):
+        dd.io._meta_from_array(x, columns=['a', 'b'])
 
 
-def test_dummy_from_recarray():
-    x = np.array([(i, i*10) for i in range(10)],
+def test_meta_from_recarray():
+    x = np.array([(i, i * 10) for i in range(10)],
                  dtype=[('a', np.float64), ('b', np.int64)])
-    res = dd.io._dummy_from_array(x)
+    res = dd.io._meta_from_array(x)
     assert isinstance(res, pd.DataFrame)
     assert res['a'].dtype == np.float64
     assert res['b'].dtype == np.int64
     tm.assert_index_equal(res.columns, pd.Index(['a', 'b']))
 
-    res = dd.io._dummy_from_array(x, columns=['x', 'y'])
+    res = dd.io._meta_from_array(x, columns=['b', 'a'])
     assert isinstance(res, pd.DataFrame)
-    assert res['x'].dtype == np.float64
-    assert res['y'].dtype == np.int64
-    tm.assert_index_equal(res.columns, pd.Index(['x', 'y']))
+    assert res['a'].dtype == np.float64
+    assert res['b'].dtype == np.int64
+    tm.assert_index_equal(res.columns, pd.Index(['b', 'a']))
 
-    msg = r"""Length mismatch: Expected axis has 2 elements, new values have 3 elements"""
-    with tm.assertRaisesRegexp(ValueError, msg):
-        dd.io._dummy_from_array(x, columns=['a', 'b', 'c'])
+    with pytest.raises(ValueError):
+        dd.io._meta_from_array(x, columns=['a', 'b', 'c'])
 
 
 def test_from_array():
     x = np.arange(10 * 3).reshape(10, 3)
     d = dd.from_array(x, chunksize=4)
     assert isinstance(d, dd.DataFrame)
-    assert d._known_dtype
     tm.assert_index_equal(d.columns, pd.Index([0, 1, 2]))
     assert d.divisions == (0, 4, 8, 9)
     assert (d.compute().values == x).all()
 
     d = dd.from_array(x, chunksize=4, columns=list('abc'))
     assert isinstance(d, dd.DataFrame)
-    assert d._known_dtype
     tm.assert_index_equal(d.columns, pd.Index(['a', 'b', 'c']))
     assert d.divisions == (0, 4, 8, 9)
     assert (d.compute().values == x).all()
@@ -222,11 +213,10 @@ def test_from_array():
 
 
 def test_from_array_with_record_dtype():
-    x = np.array([(i, i*10) for i in range(10)],
+    x = np.array([(i, i * 10) for i in range(10)],
                  dtype=[('a', 'i4'), ('b', 'i4')])
     d = dd.from_array(x, chunksize=4)
     assert isinstance(d, dd.DataFrame)
-    assert d._known_dtype
     assert list(d.columns) == ['a', 'b']
     assert d.divisions == (0, 4, 8, 9)
 
@@ -271,7 +261,6 @@ def test_from_bcolz():
     t = bcolz.ctable([[1, 2, 3], [1., 2., 3.], ['a', 'b', 'a']],
                      names=['x', 'y', 'a'])
     d = dd.from_bcolz(t, chunksize=2)
-    assert d._known_dtype
     assert d.npartitions == 2
     assert str(d.dtypes['a']) == 'category'
     assert list(d.x.compute(get=get_sync)) == [1, 2, 3]
@@ -349,7 +338,7 @@ def test_skipinitialspace():
     """)
 
     with filetext(text) as fn:
-        df = dd.read_csv(fn, skipinitialspace=True, chunkbytes=20)
+        df = dd.read_csv(fn, skipinitialspace=True, blocksize=20)
 
         assert 'amount' in df.columns
         assert df.amount.max().compute() == 600
@@ -374,7 +363,7 @@ def test_consistent_dtypes_2():
             f.write(text1)
         with open('_foo.2.csv', 'w') as f:
             f.write(text2)
-        df = dd.read_csv('_foo.*.csv', chunkbytes=25)
+        df = dd.read_csv('_foo.*.csv', blocksize=25)
 
         assert df.amount.max().compute() == 600
     finally:
@@ -413,7 +402,7 @@ def test_from_pandas_dataframe():
     ddf = dd.from_pandas(df, 3)
     assert len(ddf.dask) == 3
     assert len(ddf.divisions) == len(ddf.dask) + 1
-    assert type(ddf.divisions[0]) == type(df.index[0])
+    assert isinstance(ddf.divisions[0], type(df.index[0]))
     tm.assert_frame_equal(df, ddf.compute())
     ddf = dd.from_pandas(df, chunksize=8)
     msg = 'Exactly one of npartitions and chunksize must be specified.'
@@ -423,7 +412,7 @@ def test_from_pandas_dataframe():
         dd.from_pandas(df)
     assert len(ddf.dask) == 3
     assert len(ddf.divisions) == len(ddf.dask) + 1
-    assert type(ddf.divisions[0]) == type(df.index[0])
+    assert isinstance(ddf.divisions[0], type(df.index[0]))
     tm.assert_frame_equal(df, ddf.compute())
 
 
@@ -440,6 +429,16 @@ def test_from_pandas_small():
         assert a.divisions[0] == 0
         assert a.divisions[-1] == 2
 
+    for sort in [True, False]:
+        for i in [0, 2]:
+            df = pd.DataFrame({'x': [0] * i})
+            ddf = dd.from_pandas(df, npartitions=5, sort=sort)
+            eq(df, ddf)
+
+            s = pd.Series([0] * i, name='x')
+            ds = dd.from_pandas(s, npartitions=5, sort=sort)
+            eq(s, ds)
+
 
 @pytest.mark.xfail(reason="")
 def test_from_pandas_npartitions_is_accurate():
@@ -456,13 +455,13 @@ def test_from_pandas_series():
     ds = dd.from_pandas(s, 3)
     assert len(ds.dask) == 3
     assert len(ds.divisions) == len(ds.dask) + 1
-    assert type(ds.divisions[0]) == type(s.index[0])
+    assert isinstance(ds.divisions[0], type(s.index[0]))
     tm.assert_series_equal(s, ds.compute())
 
     ds = dd.from_pandas(s, chunksize=8)
     assert len(ds.dask) == 3
     assert len(ds.divisions) == len(ds.dask) + 1
-    assert type(ds.divisions[0]) == type(s.index[0])
+    assert isinstance(ds.divisions[0], type(s.index[0]))
     tm.assert_series_equal(s, ds.compute())
 
 
@@ -522,11 +521,10 @@ def test_Series_from_dask_array():
 def test_from_dask_array_compat_numpy_array():
     x = da.ones((3, 3, 3), chunks=2)
 
-    msg = r"from_array does not input more than 2D array, got array with shape \(3, 3, 3\)"
-    with tm.assertRaisesRegexp(ValueError, msg):
+    with pytest.raises(ValueError):
         from_dask_array(x)       # dask
 
-    with tm.assertRaisesRegexp(ValueError, msg):
+    with pytest.raises(ValueError):
         from_array(x.compute())  # numpy
 
     x = da.ones((10, 3), chunks=(3, 3))
@@ -540,11 +538,10 @@ def test_from_dask_array_compat_numpy_array():
     assert (d2.compute().values == x.compute()).all()
     tm.assert_index_equal(d2.columns, pd.Index([0, 1, 2]))
 
-    msg = r"""Length mismatch: Expected axis has 3 elements, new values have 1 elements"""
-    with tm.assertRaisesRegexp(ValueError, msg):
+    with pytest.raises(ValueError):
         from_dask_array(x, columns=['a'])       # dask
 
-    with tm.assertRaisesRegexp(ValueError, msg):
+    with pytest.raises(ValueError):
         from_array(x.compute(), columns=['a'])  # numpy
 
     d1 = from_dask_array(x, columns=['a', 'b', 'c'])       # dask
@@ -605,9 +602,10 @@ def test_from_dask_array_struct_dtype():
 
 
 def test_to_castra():
-    pytest.importorskip('castra')
+    castra = pytest.importorskip('castra')
     blosc = pytest.importorskip('blosc')
-    if LooseVersion(blosc.__version__) == '1.3.0':
+    if (LooseVersion(blosc.__version__) == '1.3.0' or
+            LooseVersion(castra.__version__) < '0.1.8'):
         pytest.skip()
     df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
                        'y': [2, 3, 4, 5]},
@@ -651,10 +649,12 @@ def test_to_castra():
         c1.drop()
         c2.drop()
 
+
 def test_from_castra():
-    pytest.importorskip('castra')
+    castra = pytest.importorskip('castra')
     blosc = pytest.importorskip('blosc')
-    if LooseVersion(blosc.__version__) == '1.3.0':
+    if (LooseVersion(blosc.__version__) == '1.3.0' or
+            LooseVersion(castra.__version__) < '0.1.8'):
         pytest.skip()
     df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
                        'y': [2, 3, 4, 5]},
@@ -680,9 +680,10 @@ def test_from_castra_with_selection():
 
     We used to use getitem for both column access and selections
     """
-    pytest.importorskip('castra')
+    castra = pytest.importorskip('castra')
     blosc = pytest.importorskip('blosc')
-    if LooseVersion(blosc.__version__) == '1.3.0':
+    if (LooseVersion(blosc.__version__) == '1.3.0' or
+            LooseVersion(castra.__version__) < '0.1.8'):
         pytest.skip()
     df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
                        'y': [2, 3, 4, 5]},
@@ -729,9 +730,12 @@ def test_to_hdf_multiple_nodes():
     df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
                        'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
     a = dd.from_pandas(df, 2)
-    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
-                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
-                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
+    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',
+                               'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                         'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13,
+                               14, 15, 16]},
+                        index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11.,
+                               12., 13., 14., 15., 16.])
     b = dd.from_pandas(df16, 16)
 
     # saving to multiple nodes
@@ -770,9 +774,12 @@ def test_to_hdf_multiple_files():
     df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
                        'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
     a = dd.from_pandas(df, 2)
-    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
-                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
-                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
+    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',
+                               'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                         'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
+                               15, 16]},
+                        index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11.,
+                               12., 13., 14., 15., 16.])
     b = dd.from_pandas(df16, 16)
 
     # saving to multiple files
@@ -901,9 +908,12 @@ def test_to_hdf_modes_multiple_files():
 def test_to_hdf_link_optimizations():
     """testing dask link levels is correct by calculating the depth of the dask graph"""
     pytest.importorskip('tables')
-    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
-                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
-                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
+    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',
+                               'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                         'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
+                               15, 16]},
+                        index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11.,
+                               12., 13., 14., 15., 16.])
     a = dd.from_pandas(df16, 16)
 
     # saving to multiple hdf files, no links are needed
@@ -931,16 +941,19 @@ def test_to_hdf_link_optimizations():
 @pytest.mark.slow
 def test_to_hdf_lock_delays():
     pytest.importorskip('tables')
-    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
-                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
-                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
+    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',
+                               'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                         'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
+                               15, 16]},
+                        index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11.,
+                               12., 13., 14., 15., 16.])
     a = dd.from_pandas(df16, 16)
 
     # adding artifichial delays to make sure last tasks finish first
     # that's a way to simulate last tasks finishing last
     def delayed_nop(i):
         if i[1] < 10:
-            sleep(0.1*(10-i[1]))
+            sleep(0.1 * (10 - i[1]))
         return i
 
     # saving to multiple hdf nodes
@@ -1060,12 +1073,16 @@ def test_to_hdf_process():
         eq(df, out)
 
 
-@pytest.mark.skipif(sys.version_info[:2] == (3,3), reason="Python3.3 uses pytest2.7.2, w/o warns method")
+@pytest.mark.skipif(sys.version_info[:2] == (3, 3),
+                    reason="Python3.3 uses pytest2.7.2, w/o warns method")
 def test_to_fmt_warns():
     pytest.importorskip('tables')
-    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
-                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
-                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
+    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h',
+                               'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                         'y': [1, 2, 3, 4, 5, 6, 7, 8, 9,
+                               10, 11, 12, 13, 14, 15, 16]},
+                        index=[1., 2., 3., 4., 5., 6., 7., 8., 9.,
+                               10., 11., 12., 13., 14., 15., 16.])
     a = dd.from_pandas(df16, 16)
 
     # testing warning when breaking order
@@ -1080,32 +1097,86 @@ def test_to_fmt_warns():
             a.to_csv(fn, name_function=str)
 
 
-def test_read_hdf():
+@pytest.mark.parametrize('data, compare', [
+    (pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
+                   'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.]),
+     tm.assert_frame_equal),
+    (pd.Series([1, 2, 3, 4], name='a'),
+     tm.assert_series_equal),
+])
+def test_read_hdf(data, compare):
     pytest.importorskip('tables')
-    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
-                       'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
     with tmpfile('h5') as fn:
-        df.to_hdf(fn, '/data')
+        data.to_hdf(fn, '/data')
         try:
-            dd.read_hdf(fn, 'data', chunksize=2)
+            dd.read_hdf(fn, 'data', chunksize=2, mode='r')
             assert False
         except TypeError as e:
             assert "format='table'" in str(e)
 
     with tmpfile('h5') as fn:
-        df.to_hdf(fn, '/data', format='table')
-        a = dd.read_hdf(fn, '/data', chunksize=2)
+        data.to_hdf(fn, '/data', format='table')
+        a = dd.read_hdf(fn, '/data', chunksize=2, mode='r')
         assert a.npartitions == 2
-        assert a._known_dtype
 
-        tm.assert_frame_equal(a.compute(), df)
+        compare(a.compute(), data)
 
-        tm.assert_frame_equal(
-              dd.read_hdf(fn, '/data', chunksize=2, start=1, stop=3).compute(),
-              pd.read_hdf(fn, '/data', start=1, stop=3))
+        compare(dd.read_hdf(fn, '/data', chunksize=2, start=1, stop=3,
+                            mode='r').compute(),
+                pd.read_hdf(fn, '/data', start=1, stop=3))
 
-        assert (sorted(dd.read_hdf(fn, '/data').dask) ==
-                sorted(dd.read_hdf(fn, '/data').dask))
+        assert (sorted(dd.read_hdf(fn, '/data', mode='r').dask) ==
+                sorted(dd.read_hdf(fn, '/data', mode='r').dask))
+
+
+def test_read_hdf_multiply_open():
+    """Test that we can read from a file that's already opened elsewhere in
+    read-only mode."""
+    pytest.importorskip('tables')
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
+                       'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
+    with tmpfile('h5') as fn:
+        df.to_hdf(fn, '/data', format='table')
+        with pd.HDFStore(fn, mode='r'):
+            dd.read_hdf(fn, '/data', chunksize=2, mode='r')
+
+
+def test_read_hdf_multiple():
+    pytest.importorskip('tables')
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h',
+                             'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9,
+                             10, 11, 12, 13, 14, 15, 16]},
+                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9.,
+                                   10., 11., 12., 13., 14., 15., 16.])
+    a = dd.from_pandas(df, 16)
+
+    with tmpfile('h5') as fn:
+        a.to_hdf(fn, '/data*')
+        r = dd.read_hdf(fn, '/data*', sorted_index=True)
+        assert a.npartitions == r.npartitions
+        assert a.divisions == r.divisions
+        eq(a, r)
+
+
+def test_read_hdf_start_stop_values():
+    pytest.importorskip('tables')
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
+                       'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
+    with tmpfile('h5') as fn:
+        df.to_hdf(fn, '/data', format='table')
+
+        with pytest.raises(ValueError) as e:
+            dd.read_hdf(fn, '/data', stop=10)
+        assert 'number of rows' in str(e)
+
+        with pytest.raises(ValueError) as e:
+            dd.read_hdf(fn, '/data', start=10)
+        assert 'is above or equal to' in str(e)
+
+        with pytest.raises(ValueError) as e:
+            dd.read_hdf(fn, '/data', chunksize=-1)
+        assert 'positive integer' in str(e)
 
 
 def test_to_csv():
@@ -1114,15 +1185,15 @@ def test_to_csv():
 
     for npartitions in [1, 2]:
         a = dd.from_pandas(df, npartitions)
-        with tmpfile('csv') as fn:
-            a.to_csv(fn, index=False)
-            result = dd.read_csv(fn).compute().reset_index(drop=True)
+        with tmpdir() as dn:
+            a.to_csv(dn, index=False)
+            result = dd.read_csv(os.path.join(dn, '*')).compute().reset_index(drop=True)
             eq(result, df)
 
-        with tmpfile('csv') as fn:
-            r = a.to_csv(fn, index=False, compute=False)
-            r.compute()
-            result = dd.read_csv(fn).compute().reset_index(drop=True)
+        with tmpdir() as dn:
+            r = a.to_csv(dn, index=False, compute=False)
+            compute(*r)
+            result = dd.read_csv(os.path.join(dn, '*')).compute().reset_index(drop=True)
             eq(result, df)
 
         with tmpdir() as dn:
@@ -1141,8 +1212,10 @@ def test_to_csv_multiple_files_cornercases():
             fn = os.path.join(dn, "data_*_*.csv")
             a.to_csv(fn)
 
-    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
-                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]})
+    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h',
+                               'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                         'y': [1, 2, 3, 4, 5, 6, 7, 8, 9,
+                               10, 11, 12, 13, 14, 15, 16]})
     a = dd.from_pandas(df16, 16)
     with tmpdir() as dn:
         fn = os.path.join(dn, 'data_*.csv')
@@ -1153,8 +1226,7 @@ def test_to_csv_multiple_files_cornercases():
     # test handling existing files when links are optimized out
     a = dd.from_pandas(df, 2)
     with tmpdir() as dn:
-        fn = os.path.join(dn, 'data_1.csv')
-        a.to_csv(fn, index=False)
+        a.to_csv(dn, index=False)
         fn = os.path.join(dn, 'data_*.csv')
         a.to_csv(fn, mode='w', index=False)
         result = dd.read_csv(fn).compute().reset_index(drop=True)
@@ -1163,20 +1235,12 @@ def test_to_csv_multiple_files_cornercases():
     # test handling existing files when links are optimized out
     a = dd.from_pandas(df16, 16)
     with tmpdir() as dn:
-        fn = os.path.join(dn, 'data_01.csv')
-        a.to_csv(fn, index=False)
+        a.to_csv(dn, index=False)
         fn = os.path.join(dn, 'data_*.csv')
         a.to_csv(fn, mode='w', index=False)
         result = dd.read_csv(fn).compute().reset_index(drop=True)
         eq(result, df16)
 
-    # test handling existing files when mode isn't 'w'
-    a = dd.from_pandas(df, 2)
-    with tmpdir() as dn:
-        fn = os.path.join(dn, 'data_*.csv')
-        with pytest.raises(ValueError):
-            a.to_csv(fn, mode='a')
-
 
 @pytest.mark.xfail(reason="to_csv does not support compression")
 def test_to_csv_gzip():
@@ -1191,27 +1255,28 @@ def test_to_csv_gzip():
             tm.assert_frame_equal(result, df)
 
 
+def test_to_csv_simple():
+    df0 = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
+                       'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
+    df = dd.from_pandas(df0, npartitions=2)
+    with tmpdir() as dir:
+        dir = str(dir)
+        df.to_csv(dir)
+        assert os.listdir(dir)
+        result = dd.read_csv(os.path.join(dir, '*')).compute()
+    assert (result.x.values == df0.x.values).all()
+
+
 def test_to_csv_series():
-    s = pd.Series([1, 2, 3], index=[10, 20, 30], name='foo')
-    a = dd.from_pandas(s, 2)
-    with tmpfile('csv') as fn:
-        with tmpfile('csv') as fn2:
-            a.to_csv(fn)
-            s.to_csv(fn2)
-            with open(fn) as f:
-                adata = f.read()
-            with open(fn2) as f:
-                sdata = f.read()
-
-            assert adata == sdata
-
-
-def test_read_csv_with_nrows():
-    with filetext(text) as fn:
-        f = dd.read_csv(fn, nrows=3)
-        assert list(f.columns) == ['name', 'amount']
-        assert f.npartitions == 1
-        assert eq(dd.read_csv(fn, nrows=3), pd.read_csv(fn, nrows=3))
+    df0 = pd.Series(['a', 'b', 'c', 'd'], index=[1., 2., 3., 4.])
+    df = dd.from_pandas(df0, npartitions=2)
+    with tmpdir() as dir:
+        dir = str(dir)
+        df.to_csv(dir)
+        assert os.listdir(dir)
+        result = dd.read_csv(os.path.join(dir, '*'), header=None,
+                             names=['x']).compute()
+    assert (result.x == df0).all()
 
 
 def test_read_csv_raises_on_no_files():
@@ -1260,7 +1325,7 @@ def test_read_csv_of_modified_file_has_different_name():
             os.fsync(f)
         b = dd.read_csv(fn)
 
-        assert sorted(a.dask) != sorted(b.dask)
+        assert sorted(a.dask, key=str) != sorted(b.dask, key=str)
 
 
 def test_to_bag():
@@ -1270,10 +1335,10 @@ def test_to_bag():
                      index=pd.Index([1., 2., 3., 4.], name='ind'))
     ddf = dd.from_pandas(a, 2)
 
-    assert ddf.to_bag().compute(get=get_sync) == list(a.itertuples(False))
-    assert ddf.to_bag(True).compute(get=get_sync) == list(a.itertuples(True))
-    assert ddf.x.to_bag(True).compute(get=get_sync) == list(a.x.iteritems())
-    assert ddf.x.to_bag().compute(get=get_sync) == list(a.x)
+    assert ddf.to_bag().compute() == list(a.itertuples(False))
+    assert ddf.to_bag(True).compute() == list(a.itertuples(True))
+    assert ddf.x.to_bag(True).compute() == list(a.x.iteritems())
+    assert ddf.x.to_bag().compute() == list(a.x)
 
 
 @pytest.mark.xfail(reason='we might want permissive behavior here')
@@ -1326,7 +1391,7 @@ def test_hdf_globbing():
 def test_index_col():
     with filetext(text) as fn:
         try:
-            f = dd.read_csv(fn, chunkbytes=30, index_col='name')
+            dd.read_csv(fn, blocksize=30, index_col='name')
             assert False
         except ValueError as e:
             assert 'set_index' in str(e)
@@ -1350,10 +1415,10 @@ def test_read_csv_with_datetime_index_partitions_one():
     with filetext(timeseries) as fn:
         df = pd.read_csv(fn, index_col=0, header=0, usecols=[0, 4],
                          parse_dates=['Date'])
-        # chunkbytes set to explicitly set to single chunk
+        # blocksize set to explicitly set to single chunk
         ddf = dd.read_csv(fn, header=0, usecols=[0, 4],
                           parse_dates=['Date'],
-                          chunkbytes=10000000).set_index('Date')
+                          blocksize=10000000).set_index('Date')
         eq(df, ddf)
 
         # because fn is so small, by default, this will only be one chunk
@@ -1369,7 +1434,7 @@ def test_read_csv_with_datetime_index_partitions_n():
         # because fn is so small, by default, set chunksize small
         ddf = dd.read_csv(fn, header=0, usecols=[0, 4],
                           parse_dates=['Date'],
-                          chunkbytes=400).set_index('Date')
+                          blocksize=400).set_index('Date')
         eq(df, ddf)
 
 
@@ -1395,7 +1460,7 @@ def test_encoding_gh601(encoding):
         test_df.to_csv(fn, encoding=encoding, index=False)
 
         a = pd.read_csv(fn, encoding=encoding)
-        d = dd.read_csv(fn, encoding=encoding, chunkbytes=1000)
+        d = dd.read_csv(fn, encoding=encoding, blocksize=1000)
         d = d.compute()
         d.index = range(len(d.index))
         assert eq(d, a)
@@ -1486,7 +1551,7 @@ def test_read_csv_slash_r():
     data = b'0,my\n1,data\n' * 1000 + b'2,foo\rbar'
     with filetext(data, mode='wb') as fn:
         dd.read_csv(fn, header=None, sep=',', lineterminator='\n',
-                    names=['a','b'], blocksize=200).compute(get=dask.get)
+                    names=['a', 'b'], blocksize=200).compute(get=dask.get)
 
 
 def test_read_csv_singleton_dtype():
@@ -1494,15 +1559,3 @@ def test_read_csv_singleton_dtype():
     with filetext(data, mode='wb') as fn:
         eq(pd.read_csv(fn, dtype=float),
            dd.read_csv(fn, dtype=float))
-
-
-def test_from_pandas_small():
-    for sort in [True, False]:
-        for i in [0, 2]:
-            df = pd.DataFrame({'x': [0] * i})
-            ddf = dd.from_pandas(df, npartitions=5, sort=sort)
-            eq(df, ddf)
-
-            s = pd.Series([0] * i, name='x')
-            ds = dd.from_pandas(s, npartitions=5, sort=sort)
-            eq(s, ds)
