@@ -26,10 +26,13 @@ pd = import_or_none('pandas')
 
 
 def test_normalize_function():
+
     def f1(a, b, c=1):
         pass
+
     def f2(a, b=1, c=2):
         pass
+
     def f3(a):
         pass
 
@@ -121,6 +124,19 @@ def test_tokenize_numpy_memmap():
     assert y != z
 
 
+@pytest.mark.skipif('not np')
+def test_tokenize_numpy_memmap_no_filename():
+    # GH 1562:
+    with tmpfile('.npy') as fn1, tmpfile('.npy') as fn2:
+        x = np.arange(5)
+        np.save(fn1, x)
+        np.save(fn2, x)
+
+        a = np.load(fn1, mmap_mode='r')
+        b = a + a
+        assert tokenize(b) == tokenize(b)
+
+
 def test_normalize_base():
     for i in [1, 1.1, '1', slice(1, 2, 3)]:
         assert normalize_token(i) is i
@@ -284,3 +300,20 @@ def test_visualize():
         dsk = {'a': 1, 'b': (add, 'a', 2), 'c': (mul, 'a', 1)}
         visualize(x, dsk, filename=os.path.join(d, 'mydask.png'))
         assert os.path.exists(os.path.join(d, 'mydask.png'))
+
+
+def test_use_cloudpickle_to_tokenize_functions_in__main__():
+    import sys
+    from textwrap import dedent
+
+    defn = dedent("""
+    def inc():
+        return x
+    """)
+
+    __main__ = sys.modules['__main__']
+    exec(compile(defn, '<test>', 'exec'), __main__.__dict__)
+    f = __main__.inc
+
+    t = normalize_token(f)
+    assert b'__main__' not in t
