@@ -1,23 +1,27 @@
 from __future__ import absolute_import, division, print_function
 
-from collections import Iterator
-from contextlib import contextmanager
-from errno import ENOENT
+import codecs
 import functools
+import inspect
 import io
+import math
 import os
-import sys
+import re
 import shutil
 import struct
+import sys
 import tempfile
-import inspect
-import codecs
-from sys import getdefaultencoding
+from errno import ENOENT
+from collections import Iterator
+from contextlib import contextmanager
+from importlib import import_module
 
-from .compatibility import long, getargspec, BZ2File, GzipFile, LZMAFile
+from .compatibility import (long, getargspec, BZ2File, GzipFile, LZMAFile, PY3,
+                            urlsplit, unicode)
+from .core import get_deps
 
 
-system_encoding = getdefaultencoding()
+system_encoding = sys.getdefaultencoding()
 if system_encoding == 'ascii':
     system_encoding = 'utf-8'
 
@@ -55,6 +59,17 @@ def ignoring(*exceptions):
         pass
 
 
+def import_required(mod_name, error_msg):
+    """Attempt to import a required dependency.
+
+    Raises a RuntimeError if the requested module is not available.
+    """
+    try:
+        return import_module(mod_name)
+    except ImportError:
+        raise RuntimeError(error_msg)
+
+
 @contextmanager
 def tmpfile(extension='', dir=None):
     extension = '.' + extension.lstrip('.')
@@ -73,6 +88,21 @@ def tmpfile(extension='', dir=None):
                     os.remove(filename)
 
 
+@contextmanager
+def tmpdir(dir=None):
+    dirname = tempfile.mkdtemp(dir=dir)
+
+    try:
+        yield dirname
+    finally:
+        if os.path.exists(dirname):
+            if os.path.isdir(dirname):
+                shutil.rmtree(dirname)
+            else:
+                with ignoring(OSError):
+                    os.remove(dirname)
+
+
 @contextmanager
 def filetext(text, extension='', open=open, mode='w'):
     with tmpfile(extension=extension) as filename:
@@ -111,6 +141,7 @@ class IndexCallable(object):
     4
     """
     __slots__ = 'fn',
+
     def __init__(self, fn):
         self.fn = fn
 
@@ -401,14 +432,13 @@ ONE_ARITY_BUILTINS = set([abs, all, any, bool, bytearray, bytes, callable, chr,
     classmethod, complex, dict, dir, enumerate, eval, float, format, frozenset,
     hash, hex, id, int, iter, len, list, max, min, next, oct, open, ord, range,
     repr, reversed, round, set, slice, sorted, staticmethod, str, sum, tuple,
-    type, vars, zip])
-if sys.version_info[0] == 3: # Python 3
-    ONE_ARITY_BUILTINS |= set([ascii])
-if sys.version_info[:2] != (2, 6):
-    ONE_ARITY_BUILTINS |= set([memoryview])
+    type, vars, zip, memoryview])
+if PY3:
+    ONE_ARITY_BUILTINS.add(ascii)  # noqa: F821
 MULTI_ARITY_BUILTINS = set([compile, delattr, divmod, filter, getattr, hasattr,
     isinstance, issubclass, map, pow, setattr])
 
+
 def takes_multiple_arguments(func):
     """ Does this function take multiple arguments?
 
@@ -544,6 +574,7 @@ def derived_from(original_klass, version=None, ua_args=[]):
 
         except AttributeError:
             module_name = original_klass.__module__.split('.')[0]
+
             @functools.wraps(method)
             def wrapped(*args, **kwargs):
                 msg = "Base package doesn't support '{0}'.".format(method_name)
@@ -561,11 +592,11 @@ def funcname(func, full=False):
         func = func.func
     try:
         if full:
-            return func.__qualname__
+            return func.__qualname__.strip('<>')
         else:
-            return func.__name__
+            return func.__name__.strip('<>')
     except:
-        return str(func)
+        return str(func).strip('<>')
 
 
 def ensure_bytes(s):
@@ -584,3 +615,254 @@ def ensure_bytes(s):
         return s.encode()
     raise TypeError(
             "Object %s is neither a bytes object nor has an encode method" % s)
+
+
+def digit(n, k, base):
+    """
+
+    >>> digit(1234, 0, 10)
+    4
+    >>> digit(1234, 1, 10)
+    3
+    >>> digit(1234, 2, 10)
+    2
+    >>> digit(1234, 3, 10)
+    1
+    """
+    return n // base**k % base
+
+
+def insert(tup, loc, val):
+    """
+
+    >>> insert(('a', 'b', 'c'), 0, 'x')
+    ('x', 'b', 'c')
+    """
+    L = list(tup)
+    L[loc] = val
+    return tuple(L)
+
+
+def build_name_function(max_int):
+    """ Returns a function that receives a single integer
+    and returns it as a string padded by enough zero characters
+    to align with maximum possible integer
+
+    >>> name_f = build_name_function(57)
+
+    >>> name_f(7)
+    '07'
+    >>> name_f(31)
+    '31'
+    >>> build_name_function(1000)(42)
+    '0042'
+    >>> build_name_function(999)(42)
+    '042'
+    >>> build_name_function(0)(0)
+    '0'
+    """
+    # handle corner cases max_int is 0 or exact power of 10
+    max_int += 1e-8
+
+    pad_length = int(math.ceil(math.log10(max_int)))
+
+    def name_function(i):
+        return str(i).zfill(pad_length)
+
+    return name_function
+
+
+def infer_storage_options(urlpath, inherit_storage_options=None):
+    """ Infer storage options from URL path and merge it with existing storage
+    options.
+
+    Parameters
+    ----------
+    urlpath: str or unicode
+        Either local absolute file path or URL (hdfs://namenode:8020/file.csv)
+    storage_options: dict (optional)
+        Its contents will get merged with the inferred information from the
+        given path
+
+    Returns
+    -------
+    Storage options dict.
+
+    Examples
+    --------
+    >>> infer_storage_options('/mnt/datasets/test.csv')  # doctest: +SKIP
+    {"protocol": "file", "path", "/mnt/datasets/test.csv"}
+    >>> infer_storage_options(
+    ...          'hdfs://username:pwd@node:123/mnt/datasets/test.csv?q=1',
+    ...          inherit_storage_options={'extra': 'value'})  # doctest: +SKIP
+    {"protocol": "hdfs", "username": "username", "password": "pwd",
+    "host": "node", "port": 123, "path": "/mnt/datasets/test.csv",
+    "url_query": "q=1", "extra": "value"}
+    """
+    # Handle Windows paths including disk name in this special case
+    if re.match(r'^[a-zA-Z]:\\', urlpath):
+        return {'protocol': 'file',
+                'path': urlpath}
+
+    parsed_path = urlsplit(urlpath)
+
+    inferred_storage_options = {
+        'protocol': parsed_path.scheme or 'file',
+        'path': parsed_path.path,
+    }
+
+    if parsed_path.netloc:
+        # Parse `hostname` from netloc manually because `parsed_path.hostname`
+        # lowercases the hostname which is not always desirable (e.g. in S3):
+        # https://github.com/dask/dask/issues/1417
+        inferred_storage_options['host'] = parsed_path.netloc.rsplit('@', 1)[-1].rsplit(':', 1)[0]
+        if parsed_path.port:
+            inferred_storage_options['port'] = parsed_path.port
+        if parsed_path.username:
+            inferred_storage_options['username'] = parsed_path.username
+        if parsed_path.password:
+            inferred_storage_options['password'] = parsed_path.password
+
+    if parsed_path.query:
+        inferred_storage_options['url_query'] = parsed_path.query
+    if parsed_path.fragment:
+        inferred_storage_options['url_fragment'] = parsed_path.fragment
+
+    if inherit_storage_options:
+        if set(inherit_storage_options) & set(inferred_storage_options):
+            raise KeyError("storage options (%r) and path url options (%r) "
+                           "collision is detected"
+                           % (inherit_storage_options, inferred_storage_options))
+        inferred_storage_options.update(inherit_storage_options)
+
+    return inferred_storage_options
+
+
+def dependency_depth(dsk):
+    import toolz
+
+    deps, _ = get_deps(dsk)
+
+    @toolz.memoize
+    def max_depth_by_deps(key):
+        if not deps[key]:
+            return 1
+
+        d = 1 + max(max_depth_by_deps(dep_key) for dep_key in deps[key])
+        return d
+
+    return max(max_depth_by_deps(dep_key) for dep_key in deps.keys())
+
+
+def eq_strict(a, b):
+    """Returns True if both values have the same type and are equal."""
+    if type(a) is type(b):
+        return a == b
+    return False
+
+
+def memory_repr(num):
+    for x in ['bytes', 'KB', 'MB', 'GB', 'TB']:
+        if num < 1024.0:
+            return "%3.1f %s" % (num, x)
+        num /= 1024.0
+
+
+def put_lines(buf, lines):
+    if any(not isinstance(x, unicode) for x in lines):
+        lines = [unicode(x) for x in lines]
+    buf.write('\n'.join(lines))
+
+
+hex_pattern = re.compile('[a-f]+')
+
+
+def key_split(s):
+    """
+    >>> key_split('x')
+    u'x'
+    >>> key_split('x-1')
+    u'x'
+    >>> key_split('x-1-2-3')
+    u'x'
+    >>> key_split(('x-2', 1))
+    'x'
+    >>> key_split("('x-2', 1)")
+    u'x'
+    >>> key_split('hello-world-1')
+    u'hello-world'
+    >>> key_split(b'hello-world-1')
+    u'hello-world'
+    >>> key_split('ae05086432ca935f6eba409a8ecd4896')
+    'data'
+    >>> key_split('<module.submodule.myclass object at 0xdaf372')
+    u'myclass'
+    >>> key_split(None)
+    'Other'
+    >>> key_split('x-abcdefab')  # ignores hex
+    u'x'
+    """
+    if type(s) is bytes:
+        s = s.decode()
+    if type(s) is tuple:
+        s = s[0]
+    try:
+        words = s.split('-')
+        result = words[0].lstrip("'(\"")
+        for word in words[1:]:
+            if word.isalpha() and not (len(word) == 8 and
+                                       hex_pattern.match(word) is not None):
+                result += '-' + word
+            else:
+                break
+        if len(result) == 32 and re.match(r'[a-f0-9]{32}', result):
+            return 'data'
+        else:
+            if result[0] == '<':
+                result = result.strip('<>').split()[0].split('.')[-1]
+            return result
+    except Exception:
+        return 'Other'
+
+
+_method_cache = {}
+
+
+class methodcaller(object):
+    """Return a callable object that calls the given method on its operand.
+
+    Unlike the builtin `methodcaller`, this class is serializable"""
+
+    __slots__ = ('method',)
+    func = property(lambda self: self.method)  # For `funcname` to work
+
+    def __new__(cls, method):
+        if method in _method_cache:
+            return _method_cache[method]
+        self = object.__new__(cls)
+        self.method = method
+        _method_cache[method] = self
+        return self
+
+    def __call__(self, obj, *args, **kwargs):
+        return getattr(obj, self.method)(*args, **kwargs)
+
+    def __reduce__(self):
+        return (methodcaller, (self.method,))
+
+
+class MethodCache(object):
+    """Attribute access on this object returns a methodcaller for that
+    attribute.
+
+    Examples
+    --------
+    >>> a = [1, 3, 3]
+    >>> M.count(a, 3) == a.count(3)
+    True
+    """
+    __getattr__ = staticmethod(methodcaller)
+    __dir__ = lambda self: list(_method_cache)
+
+
+M = MethodCache()
